/* BeTVOut.cpp 6-2001/4-2002
** written by:
** Rudolf Cornelissen	Email: rag.cornelissen@inter.nl.net
** Michael Pfeiffer		Email: michael.pfeiffer@utanet.at
*/

#define VERSION 120	//app version number = 0.120

#include <stdio.h>
#include <malloc.h>
#include <module.h>
#include <Drivers.h>
#include <be/interface/Screen.h>
#include <be/app/Application.h>

#include "../riva_iic/riva_iic.h"
#include "betvout.h"
#include "BT868_9.h"
#include "Transfer.h"
#include "Chrontel.h"

int main ()
{
	int monstat = 0, 					// 'nothing' connected
		bus0wired = 0, bus1wired = 0,	//no wired busses found
		bus0sane = 0, bus1sane = 0,		//no sane buses
		fd,								//driver pointer
		Version = 0xff,					//TVout chip silicon-version
		Type = 0xff,					//TVout chiptype
		option = 0,						//menuoption in non-VESA mode
		stat = B_OK;
	unsigned char 	OutMethod = 1, 		//TV output signals are Y/C and CVBS...
					ffilter = 1,		//flicker filter is default ON...
					vesamode = 0,		//VESA mode is turned off by default...
					input = 0,
					dualclone = 0;		//Singlehead output is default selected...

	//create device data struct...
	riva_data_t *ddata = new riva_data_t;

	new BApplication("application/x.vnd.BeTVOut");
	display_mode org_mode, new_mode;

	printf("\nBeTVOut V%.3f written by the BeTVOut team:\n"
			"Rudolf Cornelissen;    Email: RAG.Cornelissen@inter.NL.net\n"
			"Michael Pfeiffer;      Email: michael.pfeiffer@utanet.at\n\n",
			VERSION / 1000.0);
	fd = open ("/dev/video/riva_iic/1", O_RDWR);
	
	//check if driver could be reached...
	if (fd < 0)
	{
		printf("Could not reach 'riva_iic' driver, exiting...\n");
		return (B_OK);
	}

	//setup 'magic value' to unlock driver: 
	//(to keep compatibility & prevent crashes with older versions of driver and app...)
	ddata->drv_version = 0xff;	
	//notify driver of our version:
	//(to prevent old apps from accessing new drivers...)
	ddata->app_version = (uint16) VERSION;
	//check version of driver...
	stat = (ioctl(fd, rivaVersion, ddata));

	if (ddata->drv_version == 0xff)
	{
		printf("riva_iic driver V0.00\n");
		printf("\nIncompatible driver version (V0.23 or higher needed), exiting...\n");
		close (fd);
		return (B_OK);
	}
	//this works without driver supporting riva_data_t struct:
	//well, it did before I changed the first var-type.. :-(
	printf("riva_iic driver V%.2f\n",ddata->drv_version/100.0); 
	if (ddata->drv_version < 23)
	{
		printf("\nIncompatible driver version (V0.23 or higher needed), exiting...\n");
		close (fd);
		return (B_OK);
	}

	//(now we know the driver supports the riva_data_t struct...)
	if (stat) //if ioctl returned 'NACK'..
	{
		printf("\nBeTVOut application too old for use with this driver, exiting...\n"
				"(Version %.3f or higher needed.)\n",ddata->app_version/1000.0);
		close (fd);
		return (B_OK);
	}

	//unlock RIVA extended VGA registers...
	ioctl(fd, rivaUnlock);

	//check for one or two available (wired) IIC busses.
	//On RIVA TNT and later, two could be there...
	int RIVAexists = 0;
	printf("RIVA IICbus 0: ");
	ddata->iic.busnr = 0;
	if (!ioctl(fd, rivaChkBus, ddata))
	{
		printf("detected.\n");
		RIVAexists = 1;
		bus0wired = 1;
	}
	else 
	{
		printf("not detected.\n");
	}
	printf("RIVA IICbus 1: ");
	ddata->iic.busnr = 1;
	if (!ioctl(fd, rivaChkBus, ddata))
	{
		printf("detected.\n");
		RIVAexists = 1;
		bus1wired = 1;
	}
	else
	{
		printf("not detected.\n");
	}
	if (!RIVAexists)
	{
		printf("No RIVA IIC bus found, exiting...\n");
		close (fd);
		return(B_OK);
	}

	//find at least one sane IIC bus... (test this by sending an unused adress...)
	if (!FindSaneBus (fd, bus0wired, bus1wired, &bus0sane, &bus1sane, ddata))
	{
		printf("Unexpected results, exiting...\n");
		close (fd);
		return (B_OK);
	}


	//now find a TV-out chip...
	int	chfound = 0;
	byte chbus = 0, chaddr = 0, chversion;

	//check for BT or CX chip:
	int btfound = FindBT (fd, ddata);
	if (btfound)
	{
		//read chiptype and version number:
		if ((input = ReadType (fd, &Version, &Type, ddata)))
		{
			//something went wrong:
			if (input == 1)
			{
				//IIC error encountered:
				return (B_OK);
			}
			else
			{
				//The chip apparantly is not a TVout chip:
				btfound = 0;
			}	
		} 
		else
		{
			//no errors found:
			printf("ChipID = %d, chip version = %d: Chiptype is ",Type,Version);
			switch (Type)
			{
				case 0: printf("bt868.\n");
						break;
				case 1: printf("bt869.\n");
						break;
				case 2: printf("cx25870.\n");
						break;
				case 3: printf("cx25871.\n");
						break;
				default:printf("unknown, exiting...\n");
						close (fd);
						return(0);
			}
		}
	}		

	//check for CH chip:
	if (!btfound)
	{
		if (bus0sane)
		{
			ddata->iic.busnr = 0;
			chbus = 0;
			chfound = ch_detect(fd, chbus, &chaddr, &chversion, ddata);
		}
		if (bus1sane && !chfound)
		{
			ddata->iic.busnr = 1;
			chbus = 1;
			chfound = ch_detect(fd, chbus, &chaddr, &chversion, ddata);
		}

		if (chfound)
		{
			printf("\nFound one on bus %d, adress $%x;\n",chbus,chaddr);
			printf("Chiptype is ch%s.\n",ch_model_name(chversion));
		}
		else //no TVout chip at all...
		{
			printf("\nNo supported TVout chip found, exiting...\n");
			close (fd);
			return(B_OK);
		}
	}

	//get handle to Bscreen...
	BScreen screen;
	if (!screen.IsValid())
	{
		close(fd);
		printf("\nCould not get handle to Bscreen, exiting...\n");
		return(B_OK);
	}
	BScreen* screenptr = &screen;
	screen.GetMode(&org_mode); //'save' current mode..
	new_mode = org_mode;

	//check if we are in VESA mode, and see if RIVA is primary or secondary card:
	printf("\nChecking for primary card videomode: ");
	vesamode = checkVESAmode (fd, screenptr, new_mode, org_mode, ddata);
	if (vesamode) printf ("VESA active.\n");
	else printf ("Driver detected.\n");
	printf("(RIVA h_display: %d, ",ddata->displ_mode.displ_timing.h_display);
	printf("v_display: %d)\n",ddata->displ_mode.displ_timing.v_display);
	printf("(RIVA windowmanager framebuffer startadress offset: %d.)\n",(int) ddata->fb.fbw);

	if ((ddata->devID == 0) & (vesamode))
	{
		//we were unable to ascertain if RIVA is primary or secondary card!!
		//I hope later on it won't matter: this would be so if 'VESA primary'
		//and 'secondary' access to the RIVA would be identical...
		printf("\nWARNING:\nAssuming RIVA is secondary videocard!\n"
				"If it isn't, abort now, install nvidia driver, reboot and retry...\n");
		printf("Continue? Y/N (N): ");	
		scanf ("%c",&input);
		if (!(input == 'Y') & !(input == 'y'))
		{
			//abort.
			close(fd);
			return(B_OK);
		}
		ddata->devID = 1;//force RIVA is secondary card...
	}

	printf("\nChecking status of RIVA card: ");
	if (!ddata->devID)
		printf("Primary adapter.\n");
	else
	{
		printf("Secondary adapter.\n");
		//while we're at it, try to coldstart the RIVA!
		ioctl(fd, rivaColdStart, ddata);
		//switch to 800x600x16bit @ 60Hz
		ioctl(fd, rivaSetMode, ddata);
	}

	//RIVA programming should be universal for every chipversion, but
	//the 'horizontal sync phase' setting in the TV-out chip SHOULD be done 
	//seperately for different RIVA chips and videomodes though...
	//(for both CH and BT chips).
	//WARNING: This applies for both VESA and 'driver' (BeScreen) mode!
	printf("Vendor ID = $%x, Device ID = $%x;\n",
									ddata->dev.vendor_id, ddata->dev.device_id);
	printf("Nvidia Corporation");
	switch (ddata->dev.device_id)
	{
		case 0x0020:	printf(" RIVA TNT1 detected.\n");
						ddata->type = 1;
						break;
		case 0x0028:	printf(" RIVA TNT2 detected.\n");
						ddata->type = 2;
						break;
		case 0x002d:	printf(" RIVA TNT2-M64 detected.\n");
						ddata->type = 3;
						break;
		case 0x0101:	printf(" RIVA GeForce256 detected.\n");
						ddata->type = 1;
						break;
		case 0x0110:	printf(" RIVA GeForce2 MX(200/400) detected.\n");
						ddata->type = 1;
						break;
		default:		printf(": Untested card detected!\n");
						printf("\nPlease test BeTVOut to find the most optimal timing\n"
								"option for use with your card and then inform us of\n"
								"the results so we can automate setup for you in the\n"
								"next BeTVOut version...\n");
						printf("\nSelect chip specific timing:\n");
						printf("1. TNT1 or GeForce2 compatible;\n");
						printf("2. TNT2 compatible;\n");
						printf("3. TNT2-M64 compatible.\n");
						printf("Option: ");
						fflush(stdin);
						do scanf ("%c",&ddata->type);
						while ((ddata->type < '1') | (ddata->type > '3'));
						ddata->type -= 48; 
	}

	//What video signals should be outputted?
	//(only valid for BT, CH is unable to cross-switch outputs..)
	if (btfound)
	{
		printf("\nSelect TV output method:\n");
		printf("1. Force Y/C (and CVBS if supported by hardware);\n");
		printf("2. Force CVBS;\n");
		printf("3. Autodetect.\n");
		printf("Option: ");
		fflush(stdin);
		do scanf("%c",&OutMethod);
		while ((OutMethod < '1') | (OutMethod > '3'));
		OutMethod -= 48;
	}
	else OutMethod = 3; //CHfound: detect actual connections anyway for 'statusinfo'..

	//Give and execute options according to selected driver mode (VESA / non-VESA)...
	if ((!vesamode) & (ddata->devID == 0)) //videodriver in use, primary mode:
	{
		//issue warnings to screen...
		printf("\n\nWARNING:\n"
				"Do not switch between workspaces with different settings or use the\n"
				"'Screen Preferences' panel when a picture is output (also) on TV.\n");
		printf("This might result in your system sending harmfull signals to VGA.\n"
				"\nPress <ENTER> to continue...");
		fflush(stdin);
		scanf ("%c",&input);
	
nonVESAloop:	
		//let's find out what the user wants to do...
		printf("\nSelect an option:\n");
		printf(" 1. 640x480 NTSC testimage;\n");
		printf(" 2. 640x480 NTSC TV output;\n");
		printf(" 3. 800x600 NTSC TV output;\n");
		printf(" 4. 800x600 PAL testimage;\n");
		printf(" 5. 640x480 PAL TV output;\n");
		printf(" 6. 800x600 PAL TV output;\n");
		printf(" 7. VGA only (with original settings);\n");
		printf(" 8. Exit (with VGA only);\n");
		printf(" 9. 720x480 overscanning NTSC TV output (NTSC DVD/MPEG2 mode);\n");
		printf("10. 720x576 overscanning PAL TV output (PAL DVD/MPEG2 mode);\n");
		printf("11. 640x480 overscanning NTSC TV output (NTSC VCD/MPEG1 mode);\n");
		printf("12. 768x576 overscanning PAL TV output (PAL VCD/MPEG1 mode).\n");
		printf("Option: ");

		do
		{
			fflush(stdin);
			scanf ("%i",&option);
		}
		while ((option < 1) | (option > 12));
		input = option;

		switch (input)
		{
			case 1:
			case 4:	if (VGAonlyMode (fd, Type, btfound, chbus, chaddr, screenptr,
																	 org_mode,ddata))
					{
						close (fd);
						return (B_OK);
					}
					break;
/*
//field test:
	case 4:	for (int fieldstat, a = 0;a < 1000; a++)
			{
				if((fieldstat = ReadBT_Field(fd, ddata)) == -1)	
				{
					close (fd);
					return (B_OK);
				}
			printf("(TV output field status = %d)\n",fieldstat);
			}
			goto nonVESAloop;
			break;
//end field test.
*/
			case 2:
			case 3:
			case 5:
			case 6:
			case 9: 
			case 10:
			case 11:
			case 12://ask for 'singlehead' or 'dualhead clone' mode:
					dualclone = DualClone ();
					//ask for use of 'flicker filter':
					ffilter = UseFfilter ();
					break;
			case 8: //exit to VGA only.
					printf("\nExiting: ");
					//select 'VGA only' mode:
					if (VGAonlyMode (fd, Type, btfound, chbus, chaddr, screenptr,
																	 org_mode, ddata))
					{
						close (fd);
						return (B_OK);
					}
					close (fd);
					return(B_OK);
					break;
			default:;//nothing to be done here...
		}	

		if (((input == 5) | (input == 6) | (input == 10) | (input == 12)) & (dualclone))
			//dualhead clone mode selected with 50Hz on VGA. So:
			if (IssueWarning())
			{
				printf("Option aborted, returning to menu.\n");
				goto nonVESAloop;
			}

		//OK, let's do it...
		if (btfound)
		{
			//fixme: reset BT should be here...
			//(but: beware of the 'locked SDA' syndrome then!)
			if (KillClock_Blackout_BT (fd, Type, ddata)) return (B_OK);

			//if TV output signals is set to 'autodetect' then do it:
			if (OutMethod == 3)
			{
				monstat = ReadBT_ConnectStat (fd, ddata);
				if (monstat == -1) return (B_OK);
			}

			//'slowdown RIVA clock' if TVout is requested:
			//makes sure TVout chip is not being overclocked! (VESA640/800 is safe..)
			//NOTE: it looks like this is unnesessary after all..
			switch (input)
			{
				case 2:
				case 5:
				case 11:nonVESAsetVESA640 (screenptr, new_mode);
						break;
				case 3:
				case 6:
				case 9:
				case 10:
				case 12:nonVESAsetVESA800 (screenptr, new_mode);
						break;
				default:;//nothing to be done here...
			}

			//Manual config is non-dependent of the state of the PAL hardware input pin;
			//Also SDA lockups occur when setting EN_XCLK after autoconfig!
			//Make sure PAL_MD=0 for NTSC and PAL_MD = 1 for PAL...
			switch (input)
			{
				case 1: if (InitBT_NTSC640 (fd, ddata)) return (B_OK);
						break;
				case 2: if (InitBT_NTSC640 (fd, ddata)) return (B_OK);
						break;
				case 3: if (InitBT_NTSC800 (fd, ddata)) return (B_OK);
						break;
				case 4: if (InitBT_PAL800 (fd, ddata)) return (B_OK);
						break;
				case 5: if (InitBT_PAL640 (fd, ddata)) return (B_OK);
						break;
				case 6: if (InitBT_PAL800 (fd, ddata)) return (B_OK);
						break;
				case 9:	if (InitBT_NTSC720 (fd, ddata)) return (B_OK);
						break;
				case 10:if (InitBT_PAL720 (fd, ddata)) return (B_OK);
						break;
				case 11:if (InitBT_NTSC640_OS (fd, ddata)) return (B_OK);
						break;
				case 12:if (InitBT_PAL800_OS (fd, ddata)) return (B_OK);
						break;
				default:;//nothing to be done here...
			}	

			//modify BT Hphase signal to center TV image...
			if (SetBT_Hphase (fd, input, ddata)) return (B_OK);

			//set Macro mode 0...
			switch (input)
			{
				case 1:
				case 2:
				case 3:
				case 9:
				case 11:if (SetMacro (fd, ddata, 0, 0) != 0) return (B_OK);//NTSC
						break;
				case 4:
				case 5:
				case 6:
				case 10:
				case 12:if (SetMacro (fd, ddata, 1, 0) != 0) return (B_OK);//PAL
						break;
				default:;//nothing to be done here...				
			}

			//setup BT for TVout (filters, DAC_muxes...)
			if (SetupBTforTVout (fd, OutMethod, monstat, ffilter, ddata)) return (B_OK);
	    	printf("\nOption sent...");
		}
		else //CH found...
		{
			//reset CH or use another way to make sure all registers get set the same
			//way non-dependent of how they were set before BeTVOut was started...

			//if TV output signals is set to 'autodetect' then do it:
			//(make sure this does only gets called if CH input clock gets set below
			// its speed limits afterwards, or CH might get destroyed...)
			if ((OutMethod == 3) & (!(input == 1)) & (!(input == 4)))
			{
				monstat = ch_detect_connection(fd, chbus, chaddr, ddata);
				printf("\n(TV output monitor status = %d)\n",monstat);
			}
			//monstat value is:
			//0: Y/C and CVBS is connected
			//1: only Y/C is connected
			//6: only CVBS is connected
			//7: nothing is connected.
			//(tested with ASUS V7100 GeForce2 MX200 AGP 32Mb.)

			//'slowdown RIVA clock' if TVout is requested:
			//makes sure TVout chip is not being overclocked! (VESA640/800 is safe..)
			switch (input)
			{
				case 2:
				case 5:
				case 11:nonVESAsetVESA640 (screenptr, new_mode);
						break;
				case 3:
				case 6:
				case 9:
				case 10:
				case 12:nonVESAsetVESA800 (screenptr, new_mode);
						break;
				default:;//nothing to be done here...
			}

			//For TNT2 chips horizontal position should be shifted right by 8 pixels.
			//(all values tested on TNT2 and GeForce2 cards.)
			unsigned char hoffset = 2;		//2 x 4 pixels: TNT2 or TNT2 M64 
			if (ddata->type == 1) hoffset = 0;	//TNT1 or GeForce2

			//Because DVD modes with Chrontel are critical a special correction
			//needs to be done for these modes if RIVA is a TNT1 or GeForce2...
			//(It looks like the non-M64/M64 patch is needed, but this doesn't seem to
			// work here. The non-M64/M64 patch correction is normally visible only on
			// non-overscanning modes, so it only needs to be done there if these
			// modes will be set using BeScreen which currently isn't the case.
			// The currently used direct register programming method *is* using the
			// non-M64/M64 patch.
			// NOTE that the normally used 'hoffset' settings are set for centered TV
			// output, including for the Chrontel DVD modes. Only there we get
			// viewport problems if we don't apply the patch below:)
			if (((input == 9) | (input == 10)) & (ddata->type == 1)) hoffset = 2;

			switch (input)
			{
				case 1: break;//testimage not possible on CH
				case 2: //NTSC640 is mode 17, this is compatible to BT and RIVA timing.
						ch_rud_set_mode	(fd, chbus, chaddr,17,(0x30 + hoffset),
														0x02,0x01, ffilter, ddata);
														//mode, horpos, verpos, oflow
						break;
				case 3: //NTSC800 resembles mode 24.
						ch_rud_set_mode
							(fd, chbus, chaddr,24,(0x3e + hoffset),
														0xff,0x00, ffilter, ddata);
														//mode, horpos, verpos, oflow
						break;
				case 4: break;//testimage not possible on CH
				case 5: //PAL640 resembles mode 19.
						ch_rud_set_mode
							(fd, chbus, chaddr,19,(0x38 + hoffset),
														0x00,0x00, ffilter, ddata);
														//mode, horpos, verpos, oflow
						break;
				case 6: //PAL800 is mode 20, this is compatible to BT and RIVA timing.
						ch_rud_set_mode
							(fd, chbus, chaddr,20,(0x40 + hoffset),
														0x00,0x00, ffilter, ddata);
														//mode, horpos, verpos, oflow
						break;
				case 9:	//CH DVD MODES NOTICE:
						//The official CH DVD modes are interlaced input only, while
						//the RIVA cannot output interlaced data. This is why these
						//CH modes (25 and 26) cannot be used..  :-(
						//SO: if you want 'perfect' TV output in non-standard modes:
						//get a card with Brooktree BT868/869 chip mounted on it...

						//NTSC720 'resembles' mode 16.
						//(tested on TNT2-M64 with CH7006 only: has to be tested more..)
						//WARNING: do not change Hoffset: CH viewport is critical,
						//because we 'display' 720 pixels on a 640 pixel wide viewport.
						//(current values are (and have to be) exactly 'balanced'.)
						ch_rud_set_mode
							(fd, chbus, chaddr,16,(0x16 + hoffset),
														0x00,0x01, ffilter, ddata);
														//mode, horpos, verpos, oflow
						break;
				case 10://PAL720 resembles mode 19.
						//(tested on TNT2-M64 with CH7006 only: has to be tested more..)
						//WARNING: be carefull if Hoffset in changed: CH H_overscan is
						//critical, because we display 720 pixels in a 800 pixel
						//viewport.
						//(current values are (and have to be) exactly 'balanced'.)
						ch_rud_set_mode
							(fd, chbus, chaddr,19,(0x14 + hoffset),
														0x00,0x00, ffilter, ddata);
														//mode, horpos, verpos, oflow
						break;
				case 11://NTSC640_OS is mode 16, this is compatible to BT and RIVA timing.
						ch_rud_set_mode
							(fd, chbus, chaddr,16,(0x22 + hoffset),
														0x00,0x01, ffilter, ddata);
														//mode, horpos, verpos, oflow
						break;
				case 12://PAL768 resembles mode 19.
						ch_rud_set_mode
							(fd, chbus, chaddr,19,(0x1a + hoffset),
														0x00,0x00, ffilter, ddata);
														//mode, horpos, verpos, oflow
						break;
				default:;//nothing to be done here...
			//BTW: I checked the videosignals on a scope:
			//all used modes are all perfectly OK!

			//BTW 2: The CH700X seem to be much more forgiving than the BT chips!
			//If you change just a little in the wrong way on the BT your TV and VGA
			//image dissappear, while the CH does not suffer from these things!
			//Also the CH pixelclock may be as high as 50Mhz, while BT max is only 40Mhz..
			//The downside: The CH only supports the predefined modes, while the BT
			//is (almost) fully programmable...

			//BTW3: Above picture modes and position are tested on a
			//ASUS AGP 32Mb V7100 GeForce2 MX200 with CH7007 TVout chip,
			//and on a TNT2 card.
			}	

			//modify Hphase signal to center TV image...
			//already done in 'ch_rud_set_mode' above.

			//setup CH for TVout (filters...)
		    printf("\nOption sent...");
		}

		switch (input)//both BT and CH get here..
		{
			case 1: if (btfound)
					{
						if (DoTestImage (fd, ddata)) return (B_OK);
					   	printf(" and NTSC testimage activated.\n");
					}
					else //chfound
					{
					   	printf("\n\nSelected function not possible on this system.\n");
					}
					break;
			case 2: //reprogram RIVA for NTSC 640x480 @ 60Hz mode
					ioctl(fd, rivaNtsc640, ddata);
					printf(" and NTSC 640x480 @ 60Hz mode activated.\n");
					break;
			case 3: //reprogram RIVA for NTSC 800x600 @ 60Hz mode
					ioctl(fd, rivaNtsc800, ddata);
					printf(" and NTSC 800x600 @ 60Hz mode activated.\n");
					break;
			case 4: if (btfound)
					{
						if (DoTestImage (fd, ddata)) return (B_OK);
				   		printf(" and PAL testimage activated.\n"); 
					}
					else //chfound
					{
					   	printf("\n\nSelected function not possible on this system.\n");
					}
					break;
			case 5: //reprogram RIVA for PAL 640x480 @ 50Hz mode
					ioctl(fd, rivaPal640, ddata);
					printf(" and PAL 640x480 @ 50Hz mode activated.\n");
					break;
			case 6: //reprogram RIVA for PAL 800x600 @ 50Hz mode
					ioctl(fd, rivaPal800, ddata);
					printf(" and PAL 800x600 @ 50Hz mode activated.\n");
					break;
			case 9:	if (btfound)
					{
						//reprogram RIVA for NTSC 720x480 @ 60Hz mode:
						//(tested on TNT2 with BT869)
						new_mode.virtual_width = 720;			//H_ACTIVE
						new_mode.timing.h_display = 720;
						new_mode.virtual_height = 480;			//V_ACTIVEI
						new_mode.timing.v_display = 480;

						new_mode.timing.h_sync_start = 744;		//do not change!
						new_mode.timing.h_sync_end = 744+144;	//delta is H_sync_pulse
						new_mode.timing.h_total = 888;			//BT H_TOTAL
						new_mode.timing.v_sync_start = 490;		//set for centered sync pulse
						new_mode.timing.v_sync_end = 490+26;	//delta is V_sync_pulse
						new_mode.timing.v_total = 525;			//CH V_TOTAL
						new_mode.timing.pixel_clock = ((888 * 525 * 60) / 1000); //refresh
					}
					else //CH found
					{
						//reprogram RIVA for NTSC 720x480 @ 60Hz mode:
						//(tested on TNT2-M64 with CH7006)
						new_mode.virtual_width = 720;			//H_ACTIVE
						new_mode.timing.h_display = 720;
						new_mode.virtual_height = 480;			//V_ACTIVEI
						new_mode.timing.v_display = 480;

						new_mode.timing.h_sync_start = 744;		//do not change! (CH viewport is critical...)
						new_mode.timing.h_sync_end = 744+20;	//delta is H_sync_pulse
						new_mode.timing.h_total = 784;			//CH H_TOTAL
						new_mode.timing.v_sync_start = 490;		//set for centered sync pulse
						new_mode.timing.v_sync_end = 490+25;	//delta is V_sync_pulse
						new_mode.timing.v_total = 525;			//CH V_TOTAL
						new_mode.timing.pixel_clock = ((784 * 525 * 60) / 1000); //refresh
					}
					screen.SetMode(&new_mode, false);
					//switch RIVA to slave-mode for TVout chip:
					ioctl(fd, rivaEnableTVOut);
					printf(" and NTSC 720x480 @ 60Hz overscanning mode activated.\n");
					break;
			case 10://reprogram RIVA for PAL 720x576 @ 50Hz mode:(a bit modified for CH)
					new_mode.virtual_width = 720;			//BT H_ACTIVE
					new_mode.timing.h_display = 720;
					new_mode.virtual_height = 576;			//BT  V_ACTIVEI
					new_mode.timing.v_display = 576;
					new_mode.timing.h_sync_start = 744;		//set for centered sync pulse
					new_mode.timing.h_sync_end = 744+140;	//delta is BT H_BLANKI
					new_mode.timing.h_total = 888;			//BT H_CLKI
					new_mode.timing.v_sync_start = 579;		//set for centered sync pulse
					new_mode.timing.v_sync_end = 579+42;	//delta is BT V_BLANKI
					new_mode.timing.v_total = 625;			//BT V_LINESI
					new_mode.timing.pixel_clock = ((888 * 625 * 50) / 1000); //refresh
					screen.SetMode(&new_mode, false);
					//switch RIVA to slave-mode for TVout chip:
					ioctl(fd, rivaEnableTVOut);
					printf(" and PAL 720x576 @ 50Hz overscanning mode activated.\n");
					break;
			case 11://reprogram RIVA for NTSC 640x480 (OverScanning) @ 60Hz mode:
					new_mode.virtual_width = 640;			//BT H_ACTIVE
					new_mode.timing.h_display = 640;
					new_mode.virtual_height = 480;			//BT  V_ACTIVEI
					new_mode.timing.v_display = 480;
					new_mode.timing.h_sync_start = 744;		//set for CH/BT compatible TV output
					new_mode.timing.h_sync_end = 744+20;	//delta is BT H_BLANKI
					new_mode.timing.h_total = 784;			//BT H_CLKI
					new_mode.timing.v_sync_start = 490;		//set for centered sync pulse
					new_mode.timing.v_sync_end = 490+25;	//delta is BT V_BLANKI
					new_mode.timing.v_total = 525;			//BT V_LINESI
					new_mode.timing.pixel_clock = ((784 * 525 * 60) / 1000); //refresh
					screen.SetMode(&new_mode, false);
					//switch RIVA to slave-mode for TVout chip:
					ioctl(fd, rivaEnableTVOut);
					printf(" and NTSC 640x480 @ 60Hz overscanning mode activated.\n");
					break;
			case 12://reprogram RIVA for PAL 768x576 @ 50Hz mode:
					new_mode.virtual_width = 768;			//H_ACTIVE
					new_mode.timing.h_display = 768;
					new_mode.virtual_height = 576;			//V_ACTIVEI
					new_mode.timing.v_display = 576;
					new_mode.timing.h_sync_start = 848;		//set for centered TV output
					new_mode.timing.h_sync_end = 848+20;	//delta is BT H_BLANKI
					new_mode.timing.h_total = 944;			//BT H_CLKI
					new_mode.timing.v_sync_start = 579;		//set for centered sync pulse
					new_mode.timing.v_sync_end = 579+42;	//delta is BT V_BLANKI
					new_mode.timing.v_total = 625;			//BT V_LINESI
					new_mode.timing.pixel_clock = ((944 * 625 * 50) / 1000); //refresh
					screen.SetMode(&new_mode, false);
					//switch RIVA to slave-mode for TVout chip:
					ioctl(fd, rivaEnableTVOut);
					printf(" and PAL 768x576 @ 50Hz overscanning mode activated.\n");
					break;
			default:;//nothing to be done here...
		}	

		switch (input)
		{
			case 2:
			case 3:
			case 5:
			case 6:
			case 9:
			case 10:
			case 11:
			case 12://select 'singlehead' or 'dualhead clone' mode:
					if (dualclone)
					{
						//turn on VGA:
						ioctl(fd, rivaVgaOn);
					}
					else
					{
						//turn off VGA:
						ioctl(fd, rivaVgaOff);
					}
					break;
			case 7: //select 'VGA only' mode:
					if (VGAonlyMode (fd, Type, btfound, chbus, chaddr, screenptr,
																	 org_mode, ddata))
					{
						close (fd);
						return (B_OK);
					}
					break;
			default:;//nothing to be done here...
		}
	goto nonVESAloop;
	}//end non-VESA mode.

	else //VESA mode:
	{
	if (ddata->devID == 0) //VESA mode, primary card:
	{
		//issue warnings to screen...
		printf("\n\nWARNING:\n"
				"Make sure the screen resolution is set to 640x480 or 800x600 pixels\n"
				"in any colordepth before continuing or BeTVOut will abort.\n"
				"The selected resolution and colordepth can be output only.\n");
		printf("Continue? Y/N (N): ");	
		fflush(stdin);
		scanf ("%c",&input);
		if (!(input == 'Y') & !(input == 'y'))
		{
			close (fd);
			return (B_OK);
		}

		//check for valid display mode...
		if (ioctl(fd, rivaIsModeValid))
		{
			printf("\nIncompatible videomode active, exiting...\n");
			close (fd);
			return(B_OK);
		}
	}

VESAloop:	
		//let's find out what the user wants to do...
	if (ddata->devID == 0) //RIVA is primary card...
	{
		if (!ioctl(fd, rivaIsMode640)) //VGA is at 640x480 mode..
		{
			printf("\nSelect an option:\n");
			printf("1. 640x480 NTSC testimage;\n");
			printf("2. 640x480 NTSC TV output;\n");
			printf("3. 800x600 PAL testimage;\n");
			printf("4. 640x480 PAL TV output;\n");
			printf("5. VGA only (in current resolution and colordepth @ 60Hz);\n");
			printf("6. Exit (with VGA only).\n");
			printf("Option: ");
			fflush(stdin);
			do scanf ("%c",&input);
			while ((input < '1') | (input > '6'));
			input -= 48; 
			//translate options to 'full menu':
			switch (input)
			{
				case 3: input = 4;
						break;
				case 4:	input = 5;
						break;
				case 5:	input = 7;
						break;
				case 6:	input = 8;
						break;
				default:;//nothing to be done here...
			}	
		}
		else //VGA is at 800x600 mode..
		{
			printf("\nSelect an option:\n");
			printf("1. 640x480 NTSC testimage;\n");
			printf("2. 800x600 NTSC TV output;\n");
			printf("3. 800x600 PAL testimage;\n");
			printf("4. 800x600 PAL TV output;\n");
			printf("5. VGA only (in current resolution and colordepth @ 60Hz);\n");
			printf("6. Exit (with VGA only).\n");
			printf("Option: ");
			fflush(stdin);
			do scanf ("%c",&input);
			while ((input < '1') | (input > '6'));
			input -= 48; 
			//translate options to 'full menu':
			switch (input)
			{
				case 2: input = 3;
						break;
				case 3:	input = 4;
						break;
				case 4:	input = 6;
						break;
				case 5:	input = 7;
						break;
				case 6:	input = 8;
						break;
				default:;//nothing to be done here...
			}	
		}
/*
		The 'translated menu' is as follows for VESA mode:
		1. 640x480 NTSC testimage;\n");
		2. 640x480 NTSC TV output;\n");
		3. 800x600 NTSC TV output;\n");
		4. 800x600 PAL testimage;\n");
		5. 640x480 PAL TV output;\n");
		6. 800x600 PAL TV output;\n");
		7. VGA only (in current resolution and colordepth @ 60Hz);\n");
		8. Exit (with VGA only).\n");
*/
	}
	else //RIVA is secondary card...
	{
		printf("\nSelect an option:\n");
		printf("1. 640x480 NTSC testimage;\n");
		printf("2. 640x480 NTSC TV output;\n");
		printf("3. 800x600 NTSC TV output;\n");
		printf("4. 800x600 PAL testimage;\n");
		printf("5. 640x480 PAL TV output;\n");
		printf("6. 800x600 PAL TV output;\n");
		printf("7. VGA only (in last TV resolution and colordepth @ 60Hz);\n");
		printf("8. Exit (with VGA only).\n");
		printf("Option: ");
		fflush(stdin);
		do scanf ("%c",&input);
		while ((input < '1') | (input > '8'));
		input -= 48; 
		//translate options to 'full menu' not nessesary, already full menu...
	}

		switch (input)
		{
			case 1:
			case 4:
				if (ddata->devID == 0)//temp: secondary hangs system!
				{
					if (VesaVGAonlyMode (fd, Type, btfound, chbus, chaddr, ddata))
					{
						close (fd);
						return (B_OK);
					}
				} else printf("skipping VesaVGAonlyMode()\n");
					break;
			case 2:
			case 3:
			case 5:
			case 6: //ask for 'singlehead' or 'dualhead clone' mode:
					dualclone = DualClone ();
					//ask for use of 'flicker filter':
					ffilter = UseFfilter ();
					break;
			case 8: //exit to VGA only.

					//(if RIVA is secondary card shut it down preferably to make sure
					// we don't get the 'unable to ascertain RIVA prim./ sec.' situation)

					printf("\nExiting: ");
					//select 'VGA only' mode:
				if (ddata->devID == 0)//temp: secondary hangs system!
				{
					if (VesaVGAonlyMode (fd, Type, btfound, chbus, chaddr, ddata))
					{
						close (fd);
						return (B_OK);
					}
				} else printf("skipping VesaVGAonlyMode()\n");
					close (fd);
					return(B_OK);
					break;
			default:;//nothing to be done here...
		}	

		if (((input == 5) | (input == 6)) & (dualclone))
			//dualhead clone mode selected with 50Hz on VGA. So:
			if (IssueWarning())
			{
				printf("Option aborted, returning to menu.\n");
				goto VESAloop;
			}

		//OK, let's do it...
		if (btfound)
		{
			//fixme: reset BT should be here...
			//(but: beware of the 'locked SDA' syndrome then!)
			if (KillClock_Blackout_BT (fd, Type, ddata)) return (B_OK);

			//if TV output signals is set to 'autodetect' then do it:
			if (OutMethod == 3)
			{
				monstat = ReadBT_ConnectStat (fd, ddata);
				if (monstat == -1) return (B_OK);
			}

			//in VESA mode the pixelclock is more or less low enough not to overclock
			//the TVout chip... (so slowdown not necessary.)

			//Manual config is non-dependent of the state of the PAL hardware input pin;
			//Also SDA lockups occur when setting EN_XCLK after autoconfig!
			//Make sure PAL_MD=0 for NTSC and PAL_MD = 1 for PAL...
			switch (input)
			{
				case 1: if (InitBT_NTSC640 (fd, ddata)) return (B_OK);
						break;
				case 2: if (InitBT_NTSC640 (fd, ddata)) return (B_OK);
						break;
				case 3: if (InitBT_NTSC800 (fd, ddata)) return (B_OK);
						break;
				case 4: if (InitBT_PAL800 (fd, ddata)) return (B_OK);
						break;
				case 5: if (InitBT_PAL640 (fd, ddata)) return (B_OK);
						break;
				case 6: if (InitBT_PAL800 (fd, ddata)) return (B_OK);
						break;
				default:;//nothing to be done here...
			}	

			//modify BT Hphase signal to center TV image...
			if (SetBT_Hphase (fd, input, ddata)) return (B_OK);

			//set Macro mode 0...
			switch (input)
			{
				case 1:
				case 2:
				case 3:
				case 9:
				case 11:if (SetMacro (fd, ddata, 0, 0) != 0) return (B_OK);//NTSC
						break;
				case 4:
				case 5:
				case 6:
				case 10:
				case 12:if (SetMacro (fd, ddata, 1, 0) != 0) return (B_OK);//PAL
						break;
				default:;//nothing to be done here...				
			}

			//setup BT for TVout (filters, DAC_muxes...)
			if (SetupBTforTVout (fd, OutMethod, monstat, ffilter, ddata)) return (B_OK);
	    	printf("\nOption sent...");
		}
		else //CH found...
		{
			//reset CH or use another way to make sure all registers get set the same
			//way non-dependent of how they were set before BeTVOut was started...

			//if TV output signals is set to 'autodetect' then do it:
			//(make sure this does only gets called if CH input clock gets set below
			// its speed limits afterwards, or CH might get destroyed...)
			if ((OutMethod == 3) & (!(input == 1)) & (!(input == 4)))
			{
				monstat = ch_detect_connection(fd, chbus, chaddr, ddata);
				printf("\n(TV output monitor status = %d)\n",monstat);
			}
			//monstat value is:
			//0: Y/C and CVBS is connected
			//1: only Y/C is connected
			//6: only CVBS is connected
			//7: nothing is connected.
			//(tested with ASUS V7100 GeForce2 MX200 AGP 32Mb.)

			//in VESA mode the pixelclock is more or less low enough not to overclock
			//the TVout chip... (so slowdown not necessary.)

			//For TNT2 chips horizontal position should be shifted right by 8 pixels.
			//(all values tested on TNT2 and GeForce2 cards.)
			unsigned char hoffset = 2;		//2 x 4 pixels: TNT2 or TNT2 M64
			if (ddata->type == 1) hoffset = 0;	//TNT1 or Geforce2

			switch (input)
			{
				case 1: break;//testimage not possible?
				case 2: //NTSC640 is mode 17, this is compatible to BT and my RIVA timing.
						ch_rud_set_mode
							(fd, chbus, chaddr,17,(0x30 + hoffset),
														0x02,0x01, ffilter, ddata);
														//mode, horpos, verpos, oflow
						break;
				case 3: //NTSC800 resembles mode 24.
						ch_rud_set_mode
							(fd, chbus, chaddr,24,(0x3e + hoffset),
														0xff,0x00, ffilter, ddata);
														//mode, horpos, verpos, oflow
						break;
				case 4: break;//testimage not possible?
				case 5: //PAL640 resembles mode 19.
						ch_rud_set_mode
							(fd, chbus, chaddr,19,(0x38 + hoffset),
														0x00,0x00, ffilter, ddata);
														//mode, horpos, verpos, oflow
						break;
				case 6: //PAL800 is mode 20, this is compatibleto BT and my RIVA timing.
						ch_rud_set_mode
							(fd, chbus, chaddr,20,(0x40 + hoffset),
														0x00,0x00, ffilter, ddata);
														//mode, horpos, verpos, oflow
						break;
				default:;//nothing to be done here...
			//BTW: I checked the videosignals on a scope:
			//PAL640, NTSC640, PAL800 and NTSC800 are all perfectly OK!

			//BTW 2: The CH700X seem to be much more forgiving than the BT chips!
			//If you change just a little in the wrong way on the BT your TV and VGA
			//image dissappear, while the CH does not suffer from these things!
			//Also the CH pixelclock may be as high as 50Mhz, while BT max is only 40Mhz..

			//BTW3: Above picture modes and position are tested on a
			//ASUS AGP 32Mb V7100 GeForce2 MX200 with CH7007 TVout chip,
			//and on a TNT2 card.
			}	

			//modify Hphase signal to center TV image...
			//already done in 'ch_rud_set_mode' above.

			//setup CH for TVout (filters...)
		    printf("\nOption sent...");
		}

		switch (input)
		{
			case 1: if (btfound)
					{
						if (DoTestImage (fd, ddata)) return (B_OK);
					   	printf(" and NTSC testimage activated.\n");
					}
					else //chfound
					{
					   	printf("\n\nSelected function not possible on this system.\n");
					}
					break;
			case 2: //reprogram RIVA for NTSC 640x480 @ 60Hz mode
					ioctl(fd, rivaNtsc640, ddata);
					printf(" and NTSC 640x480 @ 60Hz mode activated.\n");
					break;
			case 3: //reprogram RIVA for NTSC 800x600 @ 60Hz mode
					ioctl(fd, rivaNtsc800, ddata);
					printf(" and NTSC 800x600 @ 60Hz mode activated.\n");
					break;
			case 4: if (btfound)
					{
						if (DoTestImage (fd, ddata)) return (B_OK);
				   		printf(" and PAL testimage activated.\n"); 
					}
					else //chfound
					{
					   	printf("\n\nSelected function not possible on this system.\n");
					}
					break;
			case 5: //reprogram RIVA for PAL 640x480 @ 50Hz mode
					ioctl(fd, rivaPal640, ddata);
					printf(" and PAL 640x480 @ 50Hz mode activated.\n");
					break;
			case 6: //reprogram RIVA for PAL 800x600 @ 50Hz mode
					ioctl(fd, rivaPal800, ddata);
					printf(" and PAL 800x600 @ 50Hz mode activated.\n");
					break;
			default:;//nothing to be done here...
		}	

		switch (input)
		{
			case 2:
			case 3:
			case 5:
			case 6: //select 'singlehead' or 'dualhead clone' mode:
					if (dualclone)
					{
						//turn on VGA:
						ioctl(fd, rivaVgaOn);
					}
					else
					{
						//turn off VGA:
						ioctl(fd, rivaVgaOff);
					}
					break;
			case 7: //select 'VGA only' mode:
					if (VesaVGAonlyMode (fd, Type, btfound, chbus, chaddr, ddata))
					{
						close (fd);
						return (B_OK);
					}
					break;
			default:;//nothing to be done here...
		}
	goto VESAloop;
	}//end VESA mode.
}//end main.

int checkVESAmode (int fd, BScreen* screenptr, display_mode new_mode,
						 				display_mode org_mode, riva_data_t *ddata)
{
	int status = B_OK;
	riva_data_t tmpddata = *ddata;

	//assume we are accessing the primary VGA card:
	ddata->devID = 0;
	//get current RIVA display mode:
	ioctl(fd, rivaGetMode,ddata);
	//(current Be Desktop card resolution already fetched..)

	//do prim/sec test:
	if ((ddata->displ_mode.displ_timing.h_display != new_mode.virtual_width) |
		(ddata->displ_mode.displ_timing.v_display != new_mode.virtual_height))
	{
		//RIVA in different resolution than Be Desktop:
		ddata->devID = 1; //also means status of RIVA card is known as of now..
	}

	//do destructive VESAmode check:
	if (new_mode.virtual_width == 640)
	{
		//Be Desktop (primary card) is at 640x480 mode...
		status = nonVESAsetVESA800 (screenptr, new_mode);
	}
	else
	{
		//Be Desktop (primary card) is NOT at 640x480 mode...
		status = nonVESAsetVESA640 (screenptr, new_mode);
	}

	//get newest (temporary) Be Desktop card resolution:
	(*screenptr).GetMode(&new_mode);
	//get newest (temporary) RIVA display mode:
	ioctl(fd, rivaGetMode,&tmpddata);

	//do prim/sec test again:
	if ((tmpddata.displ_mode.displ_timing.h_display != new_mode.virtual_width) |
		(tmpddata.displ_mode.displ_timing.v_display != new_mode.virtual_height))
	{
		//RIVA in different resolution than Be Desktop:
		ddata->devID = 1; //also means status of RIVA card is known as of now..
	}

	//restore original settings:
	(*screenptr).SetMode(&org_mode, false);
	
	if (status == B_OK) return (0);	//desktop is in driver (non-VESA) mode
	else return (1); //desktop is in VESA mode..
}//end checkVESAmode.

int nonVESAsetVESA640 (BScreen* screenptr, display_mode new_mode)
{
	//set VESA640: (hex values are exact copies of direct register programmings)
	new_mode.virtual_width = 640;
	new_mode.virtual_height = 480;
	new_mode.timing.h_display = 640;
	new_mode.timing.v_display = 480;
	
	new_mode.timing.h_sync_start = (0x53 * 8);//byte oriented!
	new_mode.timing.h_sync_end = ((0x1f + (0x53 && 0xe0)) * 8);//byte oriented!
	new_mode.timing.h_total = (0x5F * 8);//byte oriented! (val. *must* be set!)
	new_mode.timing.v_sync_start = 0x1ea;
	new_mode.timing.v_sync_end = ((0x1ea && 0xff0) + 0x00c);
	new_mode.timing.v_total = 0x20b;
	new_mode.timing.pixel_clock = ((((0x5F * 8) * 0x20b)* 60)/1000);// in kHz!

	return (*screenptr).SetMode(&new_mode, false);
}//end nonVESAsetVESA640.

int nonVESAsetVESA800 (BScreen* screenptr, display_mode new_mode)
{
	//set VESA800: (hex values are exact copies of direct register programmings)
	new_mode.virtual_width = 800;
	new_mode.virtual_height = 600;
	new_mode.timing.h_display = 800;
	new_mode.timing.v_display = 600;
	
	new_mode.timing.h_sync_start = (0x6a * 8);//byte oriented!
	new_mode.timing.h_sync_end = ((0x1a + (0x6a && 0xe0)) * 8);//byte oriented!
	new_mode.timing.h_total = (0x7f * 8);//byte oriented! (this value *must* be set!)
	new_mode.timing.v_sync_start = 0x259;
	new_mode.timing.v_sync_end = ((0x259 && 0xff0) + 0x00d);
	new_mode.timing.v_total = 0x272;
	new_mode.timing.pixel_clock = (((((0x7F) * 8) * 0x272)* 60)/1000);// in kHz!

	return (*screenptr).SetMode(&new_mode, false);
}//end nonVESAsetVESA800.

int VGAonlyMode	(int fd, int Type, int btfound, byte chbus, byte chaddr,
						BScreen* screenptr, display_mode org_mode, riva_data_t *ddata)
{
	if (btfound)
	{
		//prevent BT from being overclocked by VGA-only modes & black-out TV-out..
		if (KillClock_Blackout_BT (fd, Type, ddata)) return (1);
	}
	else //CHfound
	{
		//prevent CH from being overclocked by VGA-only modes...
		if (ch_rud_KillClock_CH (fd, chbus, chaddr, ddata)) return (1);
	}

	//turn on VGA only:
	ioctl(fd, rivaVgaOnly);
	(*screenptr).SetMode(&org_mode, false);
	printf(" and 'VGA only' mode activated.\n");
	printf("\nThe Be 'Screen Preferences' panel may now be used again.\n"
			"Also you may now switch between workspaces again.\n");

	if (btfound)
	{
		//a full BT reset should be done here...
	}
	else //CHfound
	{
		//reset CH: both CVBS and Y/C should be disabled also...
		//(reset may only be done AFTER VGA switchback or system might 'hang'!!)
		if (ch_rud_Reset_CH (fd, chbus, chaddr, ddata)) return (1);
	}
	return (0);//OK status
}//end VGAonlyMode.

int VesaVGAonlyMode (int fd, int Type, int btfound, byte chbus, byte chaddr, riva_data_t *ddata)
{
	if (btfound)
	{
		//prevent BT from being overclocked by VGA-only modes & black-out TV-out..
		if (KillClock_Blackout_BT (fd, Type, ddata)) return (1);
	}
	else //CHfound
	{
		//prevent CH from being overclocked by VGA-only modes...
		if (ch_rud_KillClock_CH (fd, chbus, chaddr, ddata)) return (1);
	}

	//turn on VGA only:
	ioctl(fd, rivaVesaVgaOnly);
	printf(" and 'VGA only' mode activated @ 60Hz.\n");

	if (btfound)
	{
		//a full BT reset should be done here...
	}
	else //CHfound
	{
		//reset CH: both CVBS and Y/C should be disabled also...
		//(reset may only be done AFTER VGA switchback or system might 'hang'!!)
		if (ch_rud_Reset_CH (fd, chbus, chaddr, ddata)) return (1);
	}
	return (0);//OK status
}//end VesaVGAonlyMode.

int DualClone ()
{
	unsigned char dualclone;
	int status = 0;

	//bug? in compiler: scanf simply won't work here: (the last <RET> is in there?)
	scanf("%c",&dualclone);//dummy scanf: apparantly the input 'stream' has to be clrd..

	printf("\nSelect 'Singlehead' or 'Dualhead clone' (hack) mode S/D (S): ");
	fflush(stdin);
	scanf ("%c",&dualclone);
	if ((dualclone == 'D') | (dualclone == 'd')) status = 1;
	return (status);
}//end DualClone.

int UseFfilter ()
{
	unsigned char input;

	fflush(stdin);
	printf("Do you want 'Flicker filtering'? Y/N (Y): ");
	scanf("%c",&input);
	if ((input == 'N') | (input == 'n')) return (0);
	else return (1);
}

int UseOscan ()
{
	unsigned char input;

	fflush(stdin);
	printf("Do you want 'Overscan compensation'? Y/N (Y): ");
	scanf("%c",&input);
	if ((input == 'N') | (input == 'n')) return (0);
	else return (1);
}

int IssueWarning ()
{
	unsigned char input = 0;
	int status = 0;

	//bug? in compiler: scanf simply won't work here: (the last <RET> is in there?)
	scanf("%c",&input);//dummy scanf: apparantly the input 'stream' has to be clrd..

	//issue warning to screen...
	printf("\nWARNING!!!! Before you proceed READ THIS:\nBefore enabling the 50Hz"
		" on VGA mode(s) be sure your monitor supports this.\nIf it does not it might"
		" work but get damaged anyway if you continue!\n\nYOU USE THIS"
		" PROGRAM AT YOUR OWN RISK!!!\n\n");
	printf("Continue? Y/N (N): ");	
	scanf ("%c",&input);
	if (!(input == 'Y') & !(input == 'y'))
	{
		//abort.
		status = 1;
	}
	return (status);
}//end IssueWarning.

/*
// ONLY FOR TEST PURPOSES:
static uint32 get_bpp(uint32 cs) {
	uint32	bpp = 0;

	switch (cs) {
		case B_RGB32_BIG:
		case B_RGBA32_BIG:
		case B_RGB32_LITTLE:
		case B_RGBA32_LITTLE:
			bpp = 32; break;
		case B_RGB24_BIG:
		case B_RGB24_LITTLE:
			bpp = 24; break;
		case B_RGB16_BIG:
		case B_RGB16_LITTLE:
			bpp = 16; break;
		case B_RGB15_BIG:
		case B_RGBA15_BIG:
		case B_RGB15_LITTLE:
		case B_RGBA15_LITTLE:
			bpp = 15; break;
		case B_CMAP8:
			bpp = 8; break;
	}
	return bpp;
}

void screen_test()
{
	return; // do nothing
	new BApplication("application/x.vnd.BeTVOut");
	BScreen screen;
	if (screen.IsValid()) {
		display_mode *modes;
		uint32 count;
		if (B_OK == screen.GetModeList(&modes, &count)) {
			printf("Screen Mode List:\n");
			for (unsigned int i = 0; i < count; i++) {
				#define PRINT_FLAG(flag) if ((flags & flag) != 0) printf(#flag " ")
				uint32 bpp = get_bpp(modes[i].space);
				
				printf("%dbpp %d %d %d %d ", bpp, modes[i].virtual_width, modes[i].virtual_height,
				  modes[i].h_display_start, modes[i].v_display_start);
				uint32 flags = modes[i].flags;
				PRINT_FLAG(B_SCROLL);
				PRINT_FLAG(B_8_BIT_DAC);
				PRINT_FLAG(B_HARDWARE_CURSOR);
				PRINT_FLAG(B_PARALLEL_ACCESS);
				PRINT_FLAG(B_DPMS);
				PRINT_FLAG(B_IO_FB_NA);
				if (flags != 0) printf("\n");
				
				display_timing* dt = &(modes[i].timing);
				printf("%d %d %d %d %d %d %d %d %d\n",
					dt->pixel_clock, 
					dt->h_display, dt->h_sync_start, dt->h_sync_end, dt->h_total,
					dt->v_display, dt->v_sync_start, dt->v_sync_end, dt->v_total);
				flags = dt->flags;
				PRINT_FLAG(B_BLANK_PEDESTAL);
				PRINT_FLAG(B_TIMING_INTERLACED);
				PRINT_FLAG(B_POSITIVE_HSYNC);
				PRINT_FLAG(B_POSITIVE_VSYNC);
				PRINT_FLAG(B_SYNC_ON_GREEN);
				if (flags != 0) printf("\n");

				uint32 low, high;
				if (B_OK == screen.GetPixelClockLimits(&(modes[i]), &low, &high)) {
					float total_size = dt->h_total * (float)dt->v_total;
					float low_refresh  =  (low * 1000.0) / total_size;
					float high_refresh = (high * 1000.0) / total_size;
					printf("%d %d %f %f\n", low, high, low_refresh, high_refresh);
				}
				printf("\n");
				#undef PRINT_FLAG
			}
			delete modes;
		} else {
			printf("GetModeList error\n");
		}
	} else {
		printf("BScreen error!\n");
	}
	fflush(stdout);
}
*/