/*
** Copyright 2001, Michael Pfeiffer	
*/

#include "TestWindow.h"
#include <be/app/Message.h>

// Implementation of EscapeMessageFilter
filter_result EscapeMessageFilter::Filter(BMessage *msg, BHandler **target) {
	int32 key;
	if (B_OK == msg->FindInt32("key", &key) && key == 1) {
		mWindow->PostMessage(mWhat);
		return B_SKIP_MESSAGE;
	}
	return B_DISPATCH_MESSAGE;
}

TestWindow::TestWindow(uint32 space, status_t* error) :
	BWindowScreen("Fullscreen Mode Test", space, error) {
	AddCommonFilter(new EscapeMessageFilter(this, B_QUIT_REQUESTED));
	if (*error == B_OK) {
		Show();
	} else {
		PostMessage(B_QUIT_REQUESTED);
	}
}

static inline void set_pixel(int16 x, int16 y, uint8* color, uint8* fb, int16 line_width, int16 bpp) {
	switch (bpp) {
		case 8:
			fb = fb + y * line_width + x;
			fb[0] = color[0];
			break;
		case 15:
		case 16:
			fb = fb + y * line_width + 2 * x;
			fb[0] = color[0];
			fb[1] = color[1];
			break;
		case 24:
			fb = fb + y * line_width + 3 * x;
			fb[0] = color[0];
			fb[1] = color[1];
			fb[2] = color[2];
			break;
		case 32:
			fb = fb + y * line_width + 4 * x;
			fb[0] = color[0];
			fb[1] = color[1];
			fb[2] = color[2];
			fb[3] = color[3];
			break;
	}
}

void TestWindow::DrawTestImage() {
	uint8* frame_buffer = (uint8*)(CardInfo()->frame_buffer);
	int16 line_length   = FrameBufferInfo()->bytes_per_row;
	int16 bpp           = FrameBufferInfo()->bits_per_pixel;
	int16 width         = FrameBufferInfo()->display_width;
	int16 height        = FrameBufferInfo()->display_height;
	
	uint8 color[4];

	if (bpp == 8) {
		rgb_color rgb;
		// black
		rgb.red = rgb.green = rgb.blue = 0;
		SetColorList(&rgb, 0, 0);
		// white
		rgb.red = rgb.green = rgb.blue = 0xff;
		SetColorList(&rgb, 0xff, 0xff);
	}

	// clear frame buffer	
	color[0] = color[1] = color[2] = color[3] = 0;	
	for (int y = 0; y < height; y++) {
		for (int x = 0; x < width; x++) {
			set_pixel(x, y, color, frame_buffer, line_length, bpp);
		}
	}
	
	color[0] = color[1] = color[2] = color[3] = 0xff;	
	for (int y = 0; y < height; y++) {
		set_pixel(0, y, color, frame_buffer, line_length, bpp);
		set_pixel(width-1, y, color, frame_buffer, line_length, bpp);
	}
	
	for (int x = 0; x < width; x++) {
		set_pixel(x, 0, color, frame_buffer, line_length, bpp);
		set_pixel(x, height-1, color, frame_buffer, line_length, bpp);
	}
}

void TestWindow::ScreenConnected(bool connected) {
	if (connected) {
		DrawTestImage();		
	} else {
	}
}